@@ -19,6 +19,8 @@
 
 #include "fastnetmon_logic.hpp"
 
+#include "fast_endianless.hpp"
+
 #ifdef FASTNETMON_API
 
 #ifdef __GNUC__
@@ -164,6 +166,9 @@ unsigned int check_for_availible_for_processing_packets_buckets = 1;
 // Current time with pretty low precision, we use separate thread to update it
 time_t current_inaccurate_time = 0;
 
+// This is thread safe storage for captured from the wire packets for IPv4 traffic
+packet_buckets_storage_t<uint32_t> packet_buckets_ipv4_storage;
+
 // This is thread safe storage for captured from the wire packets for IPv6 traffic
 packet_buckets_storage_t<subnet_ipv6_cidr_mask_t> packet_buckets_ipv6_storage;
 
@@ -272,7 +277,7 @@ bool enable_connection_tracking = true;
 
 bool enable_afpacket_collection         = false;
 bool enable_af_xdp_collection           = false;
-bool enable_data_collection_from_mirror = true;
+bool enable_data_collection_from_mirror = false;
 bool enable_netmap_collection           = false;
 bool enable_sflow_collection            = false;
 bool enable_netflow_collection          = false;
@@ -366,8 +371,6 @@ uint64_t our_ipv6_packets = 0;
 uint64_t incoming_total_flows_speed = 0;
 uint64_t outgoing_total_flows_speed = 0;
 
-map_of_vector_counters_t SubnetVectorMap;
-
 // Network counters for IPv6
 abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t, subnet_counter_t> ipv6_subnet_counters;
 
@@ -377,6 +380,9 @@ abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t, subnet_counter_t> ipv6_host_
 // Here we store traffic per subnet
 abstract_subnet_counters_t<subnet_cidr_mask_t, subnet_counter_t> ipv4_network_counters;
 
+// Host counters for IPv4
+abstract_subnet_counters_t<uint32_t, subnet_counter_t> ipv4_host_counters;
+
 // Flow tracking structures
 map_of_vector_counters_for_flow_t SubnetVectorMapFlow;
 
@@ -386,30 +392,17 @@ int64_t netflow_ipfix_all_protocols_total_flows_speed          = 0;
 std::string sflow_raw_packet_headers_total_speed_desc = "Number of sFlow headers per second";
 int64_t sflow_raw_packet_headers_total_speed          = 0;
 
-/* End of our data structs */
-std::mutex ban_list_details_mutex;
-std::mutex ban_list_mutex;
-std::mutex flow_counter;
-
-// map for flows
-std::map<uint64_t, int> FlowCounter;
-
-// Struct for string speed per IP
-map_of_vector_counters_t SubnetVectorMapSpeed;
-
-// Struct for storing average speed per IP for specified interval
-map_of_vector_counters_t SubnetVectorMapSpeedAverage;
+std::mutex flow_counter_mutex;
 
 #ifdef GEOIP
 map_for_counters GeoIpCounter;
 #endif
 
-// IPv6 hosts
-blackhole_ban_list_t<subnet_ipv6_cidr_mask_t> ban_list_ipv6_ng;
+// Banned IPv6 hosts
+blackhole_ban_list_t<subnet_ipv6_cidr_mask_t> ban_list_ipv6;
 
-// In ddos info we store attack power and direction
-std::map<uint32_t, banlist_item_t> ban_list;
-std::map<uint32_t, std::vector<simple_packet_t>> ban_list_details;
+// Banned IPv4 hosts
+blackhole_ban_list_t<uint32_t> ban_list_ipv4;
 
 host_group_map_t host_groups;
 
@@ -910,18 +903,18 @@ bool load_configuration_file() {
 
     logging_configuration = read_logging_settings(configuration_map);
 
-    // logger << log4cpp::Priority::INFO << "We read global ban settings: " << print_ban_thresholds(global_ban_settings);
+    logger << log4cpp::Priority::INFO << "We read global ban settings: " << print_ban_thresholds(global_ban_settings);
 
     // Read host group ban settings
-    for (host_group_map_t::iterator hostgroup_itr = host_groups.begin(); hostgroup_itr != host_groups.end(); ++hostgroup_itr) {
+    for (auto hostgroup_itr = host_groups.begin(); hostgroup_itr != host_groups.end(); ++hostgroup_itr) {
         std::string host_group_name = hostgroup_itr->first;
 
-        logger << log4cpp::Priority::INFO << "We will read ban settings for " << host_group_name;
+        logger << log4cpp::Priority::DEBUG << "We will read ban settings for " << host_group_name;
 
         host_group_ban_settings_map[host_group_name] = read_ban_settings(configuration_map, host_group_name);
 
-        // logger << log4cpp::Priority::INFO << "We read " << host_group_name << " ban settings "
-        //    << print_ban_thresholds(host_group_ban_settings_map[ host_group_name ]);
+        logger << log4cpp::Priority::DEBUG << "We read " << host_group_name << " ban settings "
+            << print_ban_thresholds(host_group_ban_settings_map[ host_group_name ]);
     }
 
     if (configuration_map.count("white_list_path") != 0) {
@@ -1093,38 +1086,44 @@ void subnet_vectors_allocator(prefix_t* prefix, void* data) {
     int network_size_in_ips = pow(base, 32 - bitlen);
     // logger<< log4cpp::Priority::INFO<<"Subnet: "<<prefix->add.sin.s_addr<<" network size:
     // "<<network_size_in_ips;
-    logger << log4cpp::Priority::INFO << "I will allocate " << network_size_in_ips << " records for subnet "
-           << subnet_as_integer << " cidr mask: " << bitlen;
-
+   
     subnet_cidr_mask_t current_subnet(subnet_as_integer, bitlen);
 
+    logger << log4cpp::Priority::INFO << "I will allocate " << network_size_in_ips << " records for subnet "
+           << convert_ipv4_subnet_to_string(current_subnet);
+
     subnet_counter_t zero_map_element{};
 
-    // Initilize our counters with fill constructor
-    try {
-        SubnetVectorMap[current_subnet]             = vector_of_counters(network_size_in_ips, zero_map_element);
-        SubnetVectorMapSpeed[current_subnet]        = vector_of_counters(network_size_in_ips, zero_map_element);
-        SubnetVectorMapSpeedAverage[current_subnet] = vector_of_counters(network_size_in_ips, zero_map_element);
-    } catch (std::bad_alloc& ba) {
-        logger << log4cpp::Priority::ERROR << "Can't allocate memory for counters";
-        exit(1);
+    for (int i = 0; i < network_size_in_ips; i++) {
+        // We need to increment IP address by X but we have to do so in little endian / host byte order
+        // incrementing big endian will not work as we expect
+        uint32_t ip_as_little_endian = fast_ntoh(subnet_as_integer);
+
+        // Increment it for specific number
+        ip_as_little_endian += i;
+
+        if (enable_connection_tracking) {
+            // On creating it initializes by zeros
+            conntrack_main_struct_t zero_conntrack_main_struct{};
+
+            SubnetVectorMapFlow[ip_as_little_endian] = zero_conntrack_main_struct;
+        }
+
+        uint32_t result_ip_as_big_endian = fast_hton(ip_as_little_endian);
+
+        // logger << log4cpp::Priority::INFO << "Allocate: " << convert_ip_as_uint_to_string(result_ip_as_big_endian);
+
+        // We use big endian values as keys
+        try {
+            ipv4_host_counters.average_speed_map[result_ip_as_big_endian] = zero_map_element;
+            ipv4_host_counters.counter_map[result_ip_as_big_endian] = zero_map_element;
+        } catch (std::bad_alloc& ba) {
+            logger << log4cpp::Priority::ERROR << "Can't allocate memory for hash counters";
+            exit(1);
+        }
     }
 
-    // Initilize map element
-    SubnetVectorMapFlow[current_subnet] = vector_of_flow_counters_t(network_size_in_ips);
-
-    // On creating it initializes by zeros
-    conntrack_main_struct_t zero_conntrack_main_struct;
-    std::fill(SubnetVectorMapFlow[current_subnet].begin(), SubnetVectorMapFlow[current_subnet].end(), zero_conntrack_main_struct);
-}
-
-void zeroify_all_counters() {
-    subnet_counter_t zero_map_element{};
-
-    for (map_of_vector_counters_t::iterator itr = SubnetVectorMap.begin(); itr != SubnetVectorMap.end(); ++itr) {
-        // logger<< log4cpp::Priority::INFO<<"Zeroify "<<itr->first;
-        std::fill(itr->second.begin(), itr->second.end(), zero_map_element);
-    }
+    logger << log4cpp::Priority::INFO << "Successfully allocated " << ipv4_host_counters.average_speed_map.size() << " counters";
 }
 
 bool load_our_networks_list() {
@@ -1293,10 +1292,6 @@ bool load_our_networks_list() {
     /* Preallocate data structures */
     patricia_process(lookup_tree_ipv4, subnet_vectors_allocator);
 
-    logger << log4cpp::Priority::INFO << "We start total zerofication of counters";
-    zeroify_all_counters();
-    logger << log4cpp::Priority::INFO << "We finished zerofication";
-
     logger << log4cpp::Priority::INFO << "We loaded " << networks_list_ipv4_as_string.size()
            << " IPv4 subnets to our in-memory list of networks";
 
@@ -1768,6 +1763,9 @@ int main(int argc, char** argv) {
 
     load_our_networks_list();
 
+    // We should specify size of circular buffers here
+    packet_buckets_ipv4_storage.set_buffers_capacity(ban_details_records_count);
+
     // Set capacity for nested buffers
     packet_buckets_ipv6_storage.set_buffers_capacity(ban_details_records_count);
 
