@@ -11,16 +11,12 @@
 #include <vector>
 
 extern struct timeval graphite_thread_execution_time;
-extern map_of_vector_counters_t SubnetVectorMapSpeed;
-extern map_of_vector_counters_t SubnetVectorMapSpeedAverage;
 extern uint64_t incoming_total_flows_speed;
 extern uint64_t outgoing_total_flows_speed;
 extern abstract_subnet_counters_t<subnet_cidr_mask_t, subnet_counter_t> ipv4_network_counters;
 extern uint64_t influxdb_writes_total;
 extern uint64_t influxdb_writes_failed;
 extern abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t, subnet_counter_t> ipv6_host_counters;
-extern abstract_subnet_counters_t<subnet_cidr_mask_t, subnet_counter_t> ipv4_host_counters;
-extern abstract_subnet_counters_t<subnet_cidr_mask_t, subnet_counter_t> ipv4_remote_host_counters;
 extern std::vector<ban_settings_t> hostgroup_list_total_calculation;
 extern std::mutex hostgroup_list_total_calculation_mutex;
 extern abstract_subnet_counters_t<int64_t, subnet_counter_t> per_hostgroup_total_counters;
@@ -28,7 +24,6 @@ extern log4cpp::Category& logger;
 extern total_speed_counters_t total_counters_ipv4;
 extern total_speed_counters_t total_counters_ipv6;
 
-
 extern std::string influxdb_database;
 extern std::string influxdb_host;
 extern unsigned short int influxdb_port;
@@ -37,7 +32,7 @@ extern std::string influxdb_user;
 extern std::string influxdb_password;
 extern unsigned int influxdb_push_period;
 
-// I do this delcaration here to avoid circuclar dependencies between fastnetmon_logic and this file
+// I do this declaration here to avoid circular dependencies between fastnetmon_logic and this file
 bool get_statistics(std::vector<system_counter_t>& system_counters);
 
 // Push system counters to InfluxDB
@@ -136,8 +131,163 @@ bool push_total_traffic_counters_to_influxdb(std::string influx_database,
     return true;
 }
 
+// Simple helper function to add additional metrics easily
+void add_counter_to_influxdb(std::map<std::string, uint64_t>& plain_total_counters_map,
+                             const traffic_counter_element_t& counter,
+                             const std::string& counter_name) {
+    plain_total_counters_map[counter_name + "_packets_incoming"] = counter.in_packets;
+    plain_total_counters_map[counter_name + "_bits_incoming"]    = counter.in_bytes * 8;
+    plain_total_counters_map[counter_name + "_packets_outgoing"] = counter.out_packets;
+    plain_total_counters_map[counter_name + "_bits_outgoing"]    = counter.out_bytes * 8;
+}
+
+
+// Fills special structure which we use to export metrics into InfluxDB
+void fill_per_protocol_countres_for_influxdb(const subnet_counter_t& current_speed_element,
+                                             std::map<std::string, uint64_t>& plain_total_counters_map) {
+
+    add_counter_to_influxdb(plain_total_counters_map, current_speed_element.dropped, "dropped");
+    add_counter_to_influxdb(plain_total_counters_map, current_speed_element.fragmented, "fragmented");
+    add_counter_to_influxdb(plain_total_counters_map, current_speed_element.tcp, "tcp");
+    add_counter_to_influxdb(plain_total_counters_map, current_speed_element.tcp_syn, "tcp_syn");
+    add_counter_to_influxdb(plain_total_counters_map, current_speed_element.udp, "udp");
+    add_counter_to_influxdb(plain_total_counters_map, current_speed_element.icmp, "icmp");
+}
+
+// Fills counters for standard fixed counters
+void fill_fixed_counters_for_influxdb(const subnet_counter_t& counter,
+                                      std::map<std::string, uint64_t>& plain_total_counters_map,
+                                      bool populate_flow) {
+    fill_main_counters_for_influxdb(counter, plain_total_counters_map, populate_flow);
+
+    bool influxdb_per_protocol_counters = false;
+
+    if (influxdb_per_protocol_counters) {
+        fill_per_protocol_countres_for_influxdb(counter, plain_total_counters_map);
+    }
+
+    return;
+}
+
+
+// Push host traffic to InfluxDB
+template <typename T, typename C>
+    // Apply limitation on type of keys because we use special string conversion function inside and we must not instantiate it for other unknown types
+    requires(std::is_same_v<T, subnet_ipv6_cidr_mask_t> || std::is_same_v<T, uint32_t>) &&
+    (std::is_same_v<C, subnet_counter_t>)bool push_hosts_traffic_counters_to_influxdb(abstract_subnet_counters_t<T, C>& host_counters,
+                                                                                      const std::string& influx_database,
+                                                                                      const std::string& influx_host,
+                                                                                      const std::string& influx_port,
+                                                                                      bool enable_auth,
+                                                                                      const std::string& influx_user,
+                                                                                      const std::string& influx_password,
+                                                                                      const std::string& measurement,
+                                                                                      const std::string& tag_name) {
+
+    std::vector<std::pair<T, C>> speed_elements;
+
+    // TODO: preallocate memory here for this array to avoid memory allocations under the lock
+    host_counters.get_all_non_zero_average_speed_elements_as_pairs(speed_elements);
+
+    // Structure for InfluxDB
+    std::vector<std::pair<std::string, std::map<std::string, uint64_t>>> hosts_vector;
+
+    for (const auto& speed_element : speed_elements) {
+        std::map<std::string, uint64_t> plain_total_counters_map;
+
+        std::string client_ip_as_string;
+
+        if constexpr (std::is_same_v<T, subnet_ipv6_cidr_mask_t>) {
+            // We use pretty strange encoding here which encodes IPv6 address as subnet but
+            // then we just discard CIDR mask because it does not matter
+            client_ip_as_string = print_ipv6_address(speed_element.first.subnet_address);
+        } else if constexpr (std::is_same_v<T, uint32_t>) {
+            // We use this encoding when we use
+            client_ip_as_string = convert_ip_as_uint_to_string(speed_element.first);
+        } else {
+            logger << log4cpp::Priority::ERROR << "No match for push_hosts_traffic_counters_to_influxdb";
+            return false;
+        }
+
+        fill_fixed_counters_for_influxdb(speed_element.second, plain_total_counters_map, true);
+
+        hosts_vector.push_back(std::make_pair(client_ip_as_string, plain_total_counters_map));
+    }
+
+    // TODO: For big networks it will cause HUGE batches, it will make sense to split them in 5-10k batches
+    if (hosts_vector.size() > 0) {
+        influxdb_writes_total++;
+
+        std::string error_text;
+
+        bool result = write_batch_of_data_to_influxdb(influx_database, influx_host, influx_port, enable_auth, influx_user,
+                                                      influx_password, measurement, tag_name, hosts_vector, error_text);
+
+        if (!result) {
+            influxdb_writes_failed++;
+            logger << log4cpp::Priority::DEBUG << "InfluxDB batch operation failed for hosts_traffic with error ";
+            return false;
+        }
+    }
+
+    return true;
+}
+
+// Push network traffic to InfluxDB
+template <typename T, typename C>
+requires std::is_same_v<C, subnet_counter_t> bool
+push_network_traffic_counters_to_influxdb(abstract_subnet_counters_t<T, C>& network_counters,
+                                          const std::string& influx_database,
+                                          const std::string& influx_host,
+                                          const std::string& influx_port,
+                                          bool enable_auth,
+                                          const std::string& influx_user,
+                                          const std::string& influx_password,
+                                          const std::string& measurement,
+                                          const std::string& tag_name) {
+
+    std::vector<std::pair<T, C>> speed_elements;
+
+    // Retrieve copy of all counters
+    network_counters.get_all_non_zero_average_speed_elements_as_pairs(speed_elements);
+
+    // Structure for InfluxDB
+    std::vector<std::pair<std::string, std::map<std::string, uint64_t>>> networks_vector;
+
+    for (const auto& speed_element : speed_elements) {
+        std::map<std::string, uint64_t> plain_total_counters_map;
+
+        // This function can convert both IPv4 and IPv6 subnets to text format
+        std::string network_as_cidr_string = convert_any_subnet_to_string(speed_element.first);
+
+        fill_fixed_counters_for_influxdb(speed_element.second, plain_total_counters_map, true);
+
+        networks_vector.push_back(std::make_pair(network_as_cidr_string, plain_total_counters_map));
+    }
+
+    if (networks_vector.size() > 0) {
+        influxdb_writes_total++;
+
+        std::string error_text;
+
+        bool result = write_batch_of_data_to_influxdb(influx_database, influx_host, influx_port, enable_auth, influx_user,
+                                                      influx_password, measurement, tag_name, networks_vector, error_text);
+
+        if (!result) {
+            influxdb_writes_failed++;
+            logger << log4cpp::Priority::DEBUG << "InfluxDB batch operation failed for " << measurement << " with error " << error_text;
+            return false;
+        }
+    }
+
+    return true;
+}
+
+
 // This thread pushes data to InfluxDB
 void influxdb_push_thread() {
+    extern abstract_subnet_counters_t<uint32_t, subnet_counter_t> ipv4_host_counters;
+
     // Sleep for a half second for shift against calculation thread
     boost::this_thread::sleep(boost::posix_time::milliseconds(500));
 
@@ -176,19 +326,21 @@ void influxdb_push_thread() {
                                                 total_counters_ipv4.total_speed_average_counters, false);
 
         // Push per subnet counters to InfluxDB
-        push_network_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
-                                                  influxdb_auth, influxdb_user, influxdb_password);
+        push_network_traffic_counters_to_influxdb(ipv4_network_counters, influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
+                                                  influxdb_auth, influxdb_user, influxdb_password, "networks_traffic", "network");
 
         // Push per host counters to InfluxDB
-        push_hosts_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
-                                                influxdb_auth, influxdb_user, influxdb_password);
+        push_hosts_traffic_counters_to_influxdb(ipv4_host_counters, influxdb_database,
+                                                current_influxdb_ip_address, std::to_string(influxdb_port), influxdb_auth,
+                                                influxdb_user, influxdb_password, "hosts_traffic", "host");
 
         push_system_counters_to_influxdb(influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
                                          influxdb_auth, influxdb_user, influxdb_password);
 
         // Push per host IPv6 counters to InfluxDB
-        push_hosts_ipv6_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
-                                                     influxdb_auth, influxdb_user, influxdb_password);
+        push_hosts_traffic_counters_to_influxdb(ipv6_host_counters, influxdb_database,
+                                                current_influxdb_ip_address, std::to_string(influxdb_port), influxdb_auth,
+                                                influxdb_user, influxdb_password, "hosts_ipv6_traffic", "host");
 
         // Push total IPv6 counters
         push_total_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
@@ -197,114 +349,6 @@ void influxdb_push_thread() {
     }
 }
 
-
-// Push host traffic to InfluxDB
-bool push_hosts_ipv6_traffic_counters_to_influxdb(std::string influx_database,
-                                                  std::string influx_host,
-                                                  std::string influx_port,
-                                                  bool enable_auth,
-                                                  std::string influx_user,
-                                                  std::string influx_password) {
-    std::vector<std::pair<subnet_ipv6_cidr_mask_t, subnet_counter_t>> speed_elements;
-
-    // TODO: preallocate memory here for this array to avoid memory allocations under the lock
-    ipv6_host_counters.get_all_non_zero_average_speed_elements_as_pairs(speed_elements);
-
-    // Structure for InfluxDB
-    std::vector<std::pair<std::string, std::map<std::string, uint64_t>>> hosts_vector;
-
-    for (const auto& speed_element : speed_elements) {
-        std::map<std::string, uint64_t> plain_total_counters_map;
-
-        std::string client_ip_as_string = print_ipv6_address(speed_element.first.subnet_address);
-
-        fill_main_counters_for_influxdb(&speed_element.second, plain_total_counters_map, true);
-
-        hosts_vector.push_back(std::make_pair(client_ip_as_string, plain_total_counters_map));
-    }
-
-    // TODO: For big networks it will cause HUGE batches, it will make sense to split them in 5-10k batches
-    if (hosts_vector.size() > 0) {
-        influxdb_writes_total++;
-
-        bool result = write_batch_of_data_to_influxdb(influx_database, influx_host, influx_port, enable_auth, influx_user,
-                                                      influx_password, "hosts_ipv6_traffic", "host", hosts_vector);
-
-        if (!result) {
-            influxdb_writes_failed++;
-            logger << log4cpp::Priority::DEBUG << "InfluxDB batch operation failed for hosts_traffic";
-            return false;
-        }
-    }
-
-    return true;
-}
-
-
-// Push host traffic to InfluxDB
-bool push_hosts_traffic_counters_to_influxdb(std::string influx_database,
-                                             std::string influx_host,
-                                             std::string influx_port,
-                                             bool enable_auth,
-                                             std::string influx_user,
-                                             std::string influx_password) {
-    /* https://docs.influxdata.com/influxdb/v1.7/concepts/glossary/:
-     A collection of points in line protocol format, separated by newlines (0x0A). A batch of points may be submitted to
-     the database using a single HTTP request to the write endpoint. This makes writes via the HTTP API much more
-     performant by drastically reducing the HTTP overhead. InfluxData recommends batch sizes of 5,000-10,000 points,
-     although different use cases may be better served by significantly smaller or larger batches.
-     */
-
-    map_of_vector_counters_t* current_speed_map = &SubnetVectorMapSpeedAverage;
-
-    // Iterate over all networks
-    for (map_of_vector_counters_t::iterator itr = current_speed_map->begin(); itr != current_speed_map->end(); ++itr) {
-        std::vector<std::pair<std::string, std::map<std::string, uint64_t>>> hosts_vector;
-
-        // Iterate over all hosts in network
-        for (vector_of_counters_t::iterator vector_itr = itr->second.begin(); vector_itr != itr->second.end(); ++vector_itr) {
-            std::map<std::string, uint64_t> plain_total_counters_map;
-
-            int current_index = vector_itr - itr->second.begin();
-
-            // Convert to host order for math operations
-            uint32_t subnet_ip                     = ntohl(itr->first.subnet_address);
-            uint32_t client_ip_in_host_bytes_order = subnet_ip + current_index;
-
-            // Convert to our standard network byte order
-            uint32_t client_ip = htonl(client_ip_in_host_bytes_order);
-
-            std::string client_ip_as_string = convert_ip_as_uint_to_string(client_ip);
-
-            // Here we could have average or instantaneous speed
-            subnet_counter_t* current_speed_element = &*vector_itr;
-
-            // Skip elements with zero speed
-            if (current_speed_element->is_zero()) {
-                continue;
-            }
-
-            fill_main_counters_for_influxdb(current_speed_element, plain_total_counters_map, true);
-
-            // Key: client_ip_as_string
-            hosts_vector.push_back(std::make_pair(client_ip_as_string, plain_total_counters_map));
-        }
-
-        if (hosts_vector.size() > 0) {
-            bool result = write_batch_of_data_to_influxdb(influx_database, influx_host, influx_port, enable_auth, influx_user,
-                                                          influx_password, "hosts_traffic", "host", hosts_vector);
-
-            if (!result) {
-                influxdb_writes_failed++;
-                logger << log4cpp::Priority::DEBUG << "InfluxDB batch operation failed for hosts_traffic";
-                return false;
-            }
-        }
-    }
-
-    return true;
-}
-
 // Write batch of data for particular InfluxDB database
 bool write_batch_of_data_to_influxdb(std::string influx_database,
                                      std::string influx_host,
@@ -314,7 +358,8 @@ bool write_batch_of_data_to_influxdb(std::string influx_database,
                                      std::string influx_password,
                                      std::string measurement,
                                      std::string tag_name,
-                                     std::vector<std::pair<std::string, std::map<std::string, uint64_t>>>& hosts_vector) {
+                                     std::vector<std::pair<std::string, std::map<std::string, uint64_t>>>& hosts_vector,
+                                     std::string& error_text) {
     // Nothing to write
     if (hosts_vector.size() == 0) {
         return true;
@@ -338,44 +383,6 @@ bool write_batch_of_data_to_influxdb(std::string influx_database,
                                   buffer.str());
 }
 
-// Push per subnet traffic counters to influxDB
-bool push_network_traffic_counters_to_influxdb(std::string influx_database,
-                                               std::string influx_host,
-                                               std::string influx_port,
-                                               bool enable_auth,
-                                               std::string influx_user,
-                                               std::string influx_password) {
-
-    std::vector<std::pair<subnet_cidr_mask_t, subnet_counter_t>> speed_elements;
-    ipv4_network_counters.get_all_non_zero_average_speed_elements_as_pairs(speed_elements);
-
-    for (const auto& itr : speed_elements) {
-        std::map<std::string, uint64_t> plain_total_counters_map;
-
-        const subnet_counter_t* speed = &itr.second;
-        std::string subnet_as_string  = convert_subnet_to_string(itr.first);
-
-        fill_main_counters_for_influxdb(speed, plain_total_counters_map, false);
-
-        influxdb_writes_total++;
-
-        std::map<std::string, std::string> tags = { { "network", subnet_as_string } };
-
-        bool result = write_line_of_data_to_influxdb(influx_database, influx_host, influx_port, enable_auth, influx_user,
-                                                     influx_password, "networks_traffic", tags, plain_total_counters_map);
-
-        if (!result) {
-            influxdb_writes_failed++;
-            logger << log4cpp::Priority::DEBUG << "InfluxDB write operation failed for networks_traffic";
-            return false;
-        }
-    }
-
-
-    return true;
-}
-
-
 // Set block of data into InfluxDB
 bool write_line_of_data_to_influxdb(std::string influx_database,
                                     std::string influx_host,
@@ -397,50 +404,20 @@ bool write_line_of_data_to_influxdb(std::string influx_database,
 }
 
 // Fills special structure which we use to export metrics into InfluxDB
-void fill_per_protocol_countres_for_influxdb(const subnet_counter_t* current_speed_element,
-                                             std::map<std::string, uint64_t>& plain_total_counters_map) {
-    plain_total_counters_map["fragmented_packets_incoming"] = current_speed_element->fragmented.in_packets;
-    plain_total_counters_map["tcp_packets_incoming"]        = current_speed_element->tcp.in_packets;
-    plain_total_counters_map["tcp_syn_packets_incoming"]    = current_speed_element->tcp_syn.in_packets;
-    plain_total_counters_map["udp_packets_incoming"]        = current_speed_element->udp.in_packets;
-    plain_total_counters_map["icmp_packets_incoming"]       = current_speed_element->icmp.in_packets;
-
-    plain_total_counters_map["fragmented_bits_incoming"] = current_speed_element->fragmented.in_bytes * 8;
-    plain_total_counters_map["tcp_bits_incoming"]        = current_speed_element->tcp.in_bytes * 8;
-    plain_total_counters_map["tcp_syn_bits_incoming"]    = current_speed_element->tcp_syn.in_bytes * 8;
-    plain_total_counters_map["udp_bits_incoming"]        = current_speed_element->udp.in_bytes * 8;
-    plain_total_counters_map["icmp_bits_incoming"]       = current_speed_element->icmp.in_bytes * 8;
-
-
-    // Outgoing
-    plain_total_counters_map["fragmented_packets_outgoing"] = current_speed_element->fragmented.out_packets;
-    plain_total_counters_map["tcp_packets_outgoing"]        = current_speed_element->tcp.out_packets;
-    plain_total_counters_map["tcp_syn_packets_outgoing"]    = current_speed_element->tcp_syn.out_packets;
-    plain_total_counters_map["udp_packets_outgoing"]        = current_speed_element->udp.out_packets;
-    plain_total_counters_map["icmp_packets_outgoing"]       = current_speed_element->icmp.out_packets;
-
-    plain_total_counters_map["fragmented_bits_outgoing"] = current_speed_element->fragmented.out_bytes * 8;
-    plain_total_counters_map["tcp_bits_outgoing"]        = current_speed_element->tcp.out_bytes * 8;
-    plain_total_counters_map["tcp_syn_bits_outgoing"]    = current_speed_element->tcp_syn.out_bytes * 8;
-    plain_total_counters_map["udp_bits_outgoing"]        = current_speed_element->udp.out_bytes * 8;
-    plain_total_counters_map["icmp_bits_outgoing"]       = current_speed_element->icmp.out_bytes * 8;
-}
-
-// Fills special structure which we use to export metrics into InfluxDB
-void fill_main_counters_for_influxdb(const subnet_counter_t* current_speed_element,
+void fill_main_counters_for_influxdb(const subnet_counter_t& current_speed_element,
                                      std::map<std::string, uint64_t>& plain_total_counters_map,
                                      bool populate_flow) {
     // Prepare incoming traffic data
-    plain_total_counters_map["packets_incoming"] = current_speed_element->total.in_packets;
-    plain_total_counters_map["bits_incoming"]    = current_speed_element->total.in_bytes * 8;
+    plain_total_counters_map["packets_incoming"] = current_speed_element.total.in_packets;
+    plain_total_counters_map["bits_incoming"]    = current_speed_element.total.in_bytes * 8;
 
     // Outdoing traffic
-    plain_total_counters_map["packets_outgoing"] = current_speed_element->total.out_packets;
-    plain_total_counters_map["bits_outgoing"]    = current_speed_element->total.out_bytes * 8;
+    plain_total_counters_map["packets_outgoing"] = current_speed_element.total.out_packets;
+    plain_total_counters_map["bits_outgoing"]    = current_speed_element.total.out_bytes * 8;
 
     if (populate_flow) {
-        plain_total_counters_map["flows_incoming"] = current_speed_element->in_flows;
-        plain_total_counters_map["flows_outgoing"] = current_speed_element->out_flows;
+        plain_total_counters_map["flows_incoming"] = current_speed_element.in_flows;
+        plain_total_counters_map["flows_outgoing"] = current_speed_element.out_flows;
     }
 }
 
@@ -448,8 +425,8 @@ void fill_main_counters_for_influxdb(const subnet_counter_t* current_speed_eleme
 // Prepare string to insert data into InfluxDB
 std::string craft_line_for_influxdb_line_protocol(uint64_t unix_timestamp_nanoseconds,
                                                   std::string measurement,
-                                                  std::map<std::string, std::string>& tags,
-                                                  std::map<std::string, uint64_t>& plain_total_counters_map) {
+                                                  const std::map<std::string, std::string>& tags,
+                                                  const std::map<std::string, uint64_t>& plain_total_counters_map) {
     std::stringstream buffer;
     buffer << measurement << ",";
 
